// 
// Decompiled by Procyon v0.5.36
// 

package org.bukkit.command;

import org.bukkit.plugin.PluginDescriptionFile;
import org.bukkit.plugin.Plugin;
import org.bukkit.ChatColor;
import java.util.Arrays;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.HashMap;
import org.bukkit.Server;
import java.util.Map;

public final class SimpleCommandMap implements CommandMap
{
    private final Map<String, Command> knownCommands;
    private final Server server;
    
    public SimpleCommandMap(final Server server) {
        this.knownCommands = new HashMap<String, Command>();
        this.setDefaultCommands(this.server = server);
    }
    
    private void setDefaultCommands(final Server server) {
        this.register("bukkit", new VersionCommand("version", server));
        this.register("bukkit", new ReloadCommand("reload", server));
        this.register("bukkit", new PluginsCommand("plugins", server));
    }
    
    public void registerAll(final String fallbackPrefix, final List<Command> commands) {
        if (commands != null) {
            for (final Command c : commands) {
                this.register(fallbackPrefix, c);
            }
        }
    }
    
    private void register(final String fallbackPrefix, final Command command) {
        final List<String> names = new ArrayList<String>();
        names.add(command.getName());
        names.addAll(command.getAliases());
        for (final String name : names) {
            this.register(name, fallbackPrefix, command);
        }
    }
    
    public boolean register(String name, final String fallbackPrefix, final Command command) {
        final boolean nameInUse = this.knownCommands.get(name) != null;
        if (nameInUse) {
            name = fallbackPrefix + ":" + name;
        }
        this.knownCommands.put(name, command);
        return !nameInUse;
    }
    
    public boolean dispatch(final CommandSender sender, final String commandLine) {
        String[] args = commandLine.split(" ");
        final String sentCommandLabel = args[0].toLowerCase();
        args = Arrays.copyOfRange(args, 1, args.length);
        final Command target = this.knownCommands.get(sentCommandLabel);
        final boolean isRegisteredCommand = target != null;
        if (isRegisteredCommand) {
            try {
                target.execute(sender, sentCommandLabel, args);
            }
            catch (CommandException ex) {
                throw ex;
            }
            catch (Throwable ex2) {
                throw new CommandException("Unhandled exception executing '" + commandLine + "' in " + target, ex2);
            }
        }
        return isRegisteredCommand;
    }
    
    public void clearCommands() {
        synchronized (this) {
            this.knownCommands.clear();
            this.setDefaultCommands(this.server);
        }
    }
    
    private static class VersionCommand extends Command
    {
        private final Server server;
        
        public VersionCommand(final String name, final Server server) {
            super(name);
            this.server = server;
            this.tooltip = "Gets the version of this server including any plugins in use";
            this.usageMessage = "/version [plugin name]";
            this.setAliases(Arrays.asList("ver", "about"));
        }
        
        @Override
        public boolean execute(final CommandSender sender, final String currentAlias, final String[] args) {
            if (args.length == 0) {
                sender.sendMessage("This server is running " + ChatColor.GREEN + this.server.getName() + ChatColor.WHITE + " version " + ChatColor.GREEN + this.server.getVersion());
                sender.sendMessage("This server is also sporting some funky dev build of Bukkit!");
            }
            else {
                final StringBuilder name = new StringBuilder();
                for (final String arg : args) {
                    if (name.length() > 0) {
                        name.append(' ');
                    }
                    name.append(arg);
                }
                final Plugin plugin = this.server.getPluginManager().getPlugin(name.toString());
                if (plugin != null) {
                    final PluginDescriptionFile desc = plugin.getDescription();
                    sender.sendMessage(ChatColor.GREEN + desc.getName() + ChatColor.WHITE + " version " + ChatColor.GREEN + desc.getVersion());
                    if (desc.getDescription() != null) {
                        sender.sendMessage(desc.getDescription());
                    }
                    if (desc.getWebsite() != null) {
                        sender.sendMessage("Website: " + ChatColor.GREEN + desc.getWebsite());
                    }
                    if (!desc.getAuthors().isEmpty()) {
                        if (desc.getAuthors().size() == 1) {
                            sender.sendMessage("Author: " + this.getAuthors(desc));
                        }
                        else {
                            sender.sendMessage("Authors: " + this.getAuthors(desc));
                        }
                    }
                }
                else {
                    sender.sendMessage("This server is not running any plugin by that name.");
                    sender.sendMessage("Use /plugins to get a list of plugins.");
                }
            }
            return true;
        }
        
        private String getAuthors(final PluginDescriptionFile desc) {
            final StringBuilder result = new StringBuilder();
            final ArrayList<String> authors = desc.getAuthors();
            for (int i = 0; i < authors.size(); ++i) {
                if (result.length() > 0) {
                    result.append(ChatColor.WHITE);
                    if (i < authors.size() - 1) {
                        result.append(", ");
                    }
                    else {
                        result.append(" and ");
                    }
                }
                result.append(ChatColor.GREEN);
                result.append(authors.get(i));
            }
            return result.toString();
        }
    }
    
    private static class ReloadCommand extends Command
    {
        private final Server server;
        
        public ReloadCommand(final String name, final Server server) {
            super(name);
            this.server = server;
            this.tooltip = "Reloads the server configuration and plugins";
            this.usageMessage = "/reload";
            this.setAliases(Arrays.asList("rl"));
        }
        
        @Override
        public boolean execute(final CommandSender sender, final String currentAlias, final String[] args) {
            if (sender.isOp()) {
                this.server.reload();
                sender.sendMessage(ChatColor.GREEN + "Reload complete.");
            }
            else {
                sender.sendMessage(ChatColor.RED + "You do not have sufficient access" + " to reload this server.");
            }
            return true;
        }
    }
    
    private static class PluginsCommand extends Command
    {
        private final Server server;
        
        public PluginsCommand(final String name, final Server server) {
            super(name);
            this.server = server;
            this.tooltip = "Gets a list of plugins running on the server";
            this.usageMessage = "/plugins";
            this.setAliases(Arrays.asList("pl"));
        }
        
        @Override
        public boolean execute(final CommandSender sender, final String currentAlias, final String[] args) {
            sender.sendMessage("Plugins: " + this.getPluginList());
            return true;
        }
        
        private String getPluginList() {
            final StringBuilder pluginList = new StringBuilder();
            final Plugin[] arr$;
            final Plugin[] plugins = arr$ = this.server.getPluginManager().getPlugins();
            for (final Plugin plugin : arr$) {
                if (pluginList.length() > 0) {
                    pluginList.append(ChatColor.WHITE);
                    pluginList.append(", ");
                }
                pluginList.append(ChatColor.GREEN);
                pluginList.append(plugin.getDescription().getName());
            }
            return pluginList.toString();
        }
    }
}
