// 
// Decompiled by Procyon v0.5.36
// 

package org.bukkit.craftbukkit.scheduler;

import java.util.concurrent.CancellationException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.TimeoutException;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.Callable;
import java.util.concurrent.Future;

public class CraftFuture<T> implements Runnable, Future<T>
{
    private final CraftScheduler craftScheduler;
    private final Callable<T> callable;
    private final ObjectContainer<T> returnStore;
    private boolean done;
    private boolean running;
    private boolean cancelled;
    private Exception e;
    private int taskId;
    
    CraftFuture(final CraftScheduler craftScheduler, final Callable callable) {
        this.returnStore = new ObjectContainer<T>();
        this.done = false;
        this.running = false;
        this.cancelled = false;
        this.e = null;
        this.taskId = -1;
        this.callable = (Callable<T>)callable;
        this.craftScheduler = craftScheduler;
    }
    
    public void run() {
        synchronized (this) {
            if (this.cancelled) {
                return;
            }
            this.running = true;
        }
        try {
            this.returnStore.setObject(this.callable.call());
        }
        catch (Exception e) {
            this.e = e;
        }
        synchronized (this) {
            this.running = false;
            this.done = true;
            this.notify();
        }
    }
    
    public T get() throws InterruptedException, ExecutionException {
        try {
            return this.get(0L, TimeUnit.MILLISECONDS);
        }
        catch (TimeoutException te) {
            return null;
        }
    }
    
    public T get(final long timeout, final TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException {
        synchronized (this) {
            if (this.isDone()) {
                return this.getResult();
            }
            this.wait(TimeUnit.MILLISECONDS.convert(timeout, unit));
            return this.getResult();
        }
    }
    
    public T getResult() throws ExecutionException {
        if (this.cancelled) {
            throw new CancellationException();
        }
        if (this.e != null) {
            throw new ExecutionException(this.e);
        }
        return this.returnStore.getObject();
    }
    
    public boolean isDone() {
        synchronized (this) {
            return this.done;
        }
    }
    
    public boolean isCancelled() {
        synchronized (this) {
            return this.cancelled;
        }
    }
    
    public boolean cancel(final boolean mayInterruptIfRunning) {
        synchronized (this) {
            if (this.cancelled) {
                return false;
            }
            this.cancelled = true;
            if (this.taskId != -1) {
                this.craftScheduler.cancelTask(this.taskId);
            }
            return !this.running && !this.done;
        }
    }
    
    public void setTaskId(final int taskId) {
        synchronized (this) {
            this.taskId = taskId;
        }
    }
}
