// 
// Decompiled by Procyon v0.5.36
// 

package org.bukkit.craftbukkit.scheduler;

import java.util.Iterator;
import java.util.concurrent.Future;
import java.util.concurrent.Callable;
import org.bukkit.plugin.Plugin;
import java.util.concurrent.locks.ReentrantLock;
import java.util.concurrent.locks.Lock;
import java.util.TreeMap;
import java.util.LinkedList;
import org.bukkit.craftbukkit.CraftServer;
import org.bukkit.scheduler.BukkitScheduler;

public class CraftScheduler implements BukkitScheduler, Runnable
{
    private final CraftServer server;
    private final CraftThreadManager craftThreadManager;
    private final LinkedList<Runnable> mainThreadQueue;
    private final LinkedList<Runnable> syncedTasks;
    private final TreeMap<CraftTask, Boolean> schedulerQueue;
    private final Object currentTickSync;
    private Long currentTick;
    private final Lock mainThreadLock;
    
    public void run() {
        while (true) {
            boolean stop = false;
            long firstTick = -1L;
            long currentTick = -1L;
            CraftTask first = null;
            do {
                synchronized (this.schedulerQueue) {
                    first = null;
                    if (!this.schedulerQueue.isEmpty()) {
                        first = this.schedulerQueue.firstKey();
                        if (first != null) {
                            currentTick = this.getCurrentTick();
                            firstTick = first.getExecutionTick();
                            if (currentTick >= firstTick) {
                                this.schedulerQueue.remove(first);
                                this.processTask(first);
                                if (first.getPeriod() < 0L) {
                                    continue;
                                }
                                first.updateExecution();
                                this.schedulerQueue.put(first, first.isSync());
                            }
                            else {
                                stop = true;
                            }
                        }
                        else {
                            stop = true;
                        }
                    }
                    else {
                        stop = true;
                    }
                }
            } while (!stop);
            long sleepTime = 0L;
            if (first == null) {
                sleepTime = 60000L;
            }
            else {
                currentTick = this.getCurrentTick();
                sleepTime = (firstTick - currentTick) * 50L + 25L;
            }
            if (sleepTime < 50L) {
                sleepTime = 50L;
            }
            else if (sleepTime > 60000L) {
                sleepTime = 60000L;
            }
            synchronized (this.schedulerQueue) {
                try {
                    this.schedulerQueue.wait(sleepTime);
                }
                catch (InterruptedException ex) {}
            }
        }
    }
    
    void processTask(final CraftTask task) {
        if (task.isSync()) {
            this.addToMainThreadQueue(task.getTask());
        }
        else {
            this.craftThreadManager.executeTask(task.getTask(), task.getOwner(), task.getIdNumber());
        }
    }
    
    public CraftScheduler(final CraftServer server) {
        this.craftThreadManager = new CraftThreadManager();
        this.mainThreadQueue = new LinkedList<Runnable>();
        this.syncedTasks = new LinkedList<Runnable>();
        this.schedulerQueue = new TreeMap<CraftTask, Boolean>();
        this.currentTickSync = new Object();
        this.currentTick = 0L;
        this.mainThreadLock = new ReentrantLock();
        this.server = server;
        final Thread t = new Thread(this);
        t.start();
    }
    
    public void mainThreadHeartbeat(final long currentTick) {
        if (this.mainThreadLock.tryLock()) {
            try {
                this.currentTick = currentTick;
                while (!this.mainThreadQueue.isEmpty()) {
                    this.syncedTasks.addLast(this.mainThreadQueue.removeFirst());
                }
            }
            finally {
                this.mainThreadLock.unlock();
            }
            while (!this.syncedTasks.isEmpty()) {
                this.syncedTasks.removeFirst().run();
            }
        }
    }
    
    long getCurrentTick() {
        this.mainThreadLock.lock();
        long tempTick = 0L;
        try {
            tempTick = this.currentTick;
        }
        finally {
            this.mainThreadLock.unlock();
        }
        return tempTick;
    }
    
    void addToMainThreadQueue(final Runnable task) {
        this.mainThreadLock.lock();
        try {
            this.mainThreadQueue.addLast(task);
        }
        finally {
            this.mainThreadLock.unlock();
        }
    }
    
    public int scheduleSyncDelayedTask(final Plugin plugin, final Runnable task, final long delay) {
        return this.scheduleSyncRepeatingTask(plugin, task, delay, -1L);
    }
    
    public int scheduleSyncDelayedTask(final Plugin plugin, final Runnable task) {
        return this.scheduleSyncDelayedTask(plugin, task, 0L);
    }
    
    public int scheduleSyncRepeatingTask(final Plugin plugin, final Runnable task, final long delay, final long period) {
        final CraftTask newTask = new CraftTask(plugin, task, true, this.getCurrentTick() + delay, period);
        synchronized (this.schedulerQueue) {
            this.schedulerQueue.put(newTask, true);
            this.schedulerQueue.notify();
        }
        return newTask.getIdNumber();
    }
    
    public int scheduleAsyncDelayedTask(final Plugin plugin, final Runnable task, final long delay) {
        return this.scheduleAsyncRepeatingTask(plugin, task, delay, -1L);
    }
    
    public int scheduleAsyncDelayedTask(final Plugin plugin, final Runnable task) {
        return this.scheduleAsyncDelayedTask(plugin, task, 0L);
    }
    
    public int scheduleAsyncRepeatingTask(final Plugin plugin, final Runnable task, final long delay, final long period) {
        final CraftTask newTask = new CraftTask(plugin, task, false, this.getCurrentTick() + delay, period);
        synchronized (this.schedulerQueue) {
            this.schedulerQueue.put(newTask, false);
            this.schedulerQueue.notify();
        }
        return newTask.getIdNumber();
    }
    
    public <T> Future<T> callSyncMethod(final Plugin plugin, final Callable<T> task) {
        final CraftFuture<T> craftFuture = new CraftFuture<T>(this, task);
        synchronized (craftFuture) {
            final int taskId = this.scheduleSyncDelayedTask(plugin, craftFuture);
            craftFuture.setTaskId(taskId);
        }
        return craftFuture;
    }
    
    public void cancelTask(final int taskId) {
        synchronized (this.schedulerQueue) {
            final Iterator<CraftTask> itr = this.schedulerQueue.keySet().iterator();
            while (itr.hasNext()) {
                final CraftTask current = itr.next();
                if (current.getIdNumber() == taskId) {
                    itr.remove();
                }
            }
        }
        this.craftThreadManager.interruptTask(taskId);
    }
    
    public void cancelTasks(final Plugin plugin) {
        synchronized (this.schedulerQueue) {
            final Iterator<CraftTask> itr = this.schedulerQueue.keySet().iterator();
            while (itr.hasNext()) {
                final CraftTask current = itr.next();
                if (current.getOwner().equals(plugin)) {
                    itr.remove();
                }
            }
        }
        this.craftThreadManager.interruptTasks(plugin);
    }
    
    public void cancelAllTasks() {
        synchronized (this.schedulerQueue) {
            this.schedulerQueue.clear();
        }
        this.craftThreadManager.interruptAllTasks();
    }
    
    public boolean isCurrentlyRunning(final int taskId) {
        return this.craftThreadManager.isAlive(taskId);
    }
    
    public boolean isQueued(final int taskId) {
        synchronized (this.schedulerQueue) {
            for (final CraftTask current : this.schedulerQueue.keySet()) {
                if (current.getIdNumber() == taskId) {
                    return true;
                }
            }
            return false;
        }
    }
}
