// 
// Decompiled by Procyon v0.5.36
// 

package org.bukkit.plugin;

import java.util.TreeSet;
import org.bukkit.event.Listener;
import java.util.regex.Matcher;
import java.util.Iterator;
import java.util.Set;
import java.util.logging.Level;
import java.io.File;
import java.lang.reflect.Constructor;
import java.util.EnumMap;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Comparator;
import java.util.SortedSet;
import org.bukkit.event.Event;
import java.util.List;
import java.util.regex.Pattern;
import java.util.Map;
import org.bukkit.Server;

public final class SimplePluginManager implements PluginManager
{
    private final Server server;
    private final Map<Pattern, PluginLoader> fileAssociations;
    private final List<Plugin> plugins;
    private final Map<String, Plugin> lookupNames;
    private final Map<Event.Type, SortedSet<RegisteredListener>> listeners;
    private final Comparator<RegisteredListener> comparer;
    
    public SimplePluginManager(final Server instance) {
        this.fileAssociations = new HashMap<Pattern, PluginLoader>();
        this.plugins = new ArrayList<Plugin>();
        this.lookupNames = new HashMap<String, Plugin>();
        this.listeners = new EnumMap<Event.Type, SortedSet<RegisteredListener>>(Event.Type.class);
        this.comparer = new Comparator<RegisteredListener>() {
            public int compare(final RegisteredListener i, final RegisteredListener j) {
                int result = i.getPriority().compareTo(j.getPriority());
                if (result == 0 && i != j) {
                    result = 1;
                }
                return result;
            }
        };
        this.server = instance;
    }
    
    public void RegisterInterface(final Class<? extends PluginLoader> loader) throws IllegalArgumentException {
        if (PluginLoader.class.isAssignableFrom(loader)) {
            PluginLoader instance;
            try {
                final Constructor<? extends PluginLoader> constructor = loader.getConstructor(Server.class);
                instance = (PluginLoader)constructor.newInstance(this.server);
            }
            catch (NoSuchMethodException ex) {
                throw new IllegalArgumentException(String.format("Class %s does not have a public %s(Server) constructor", loader.getName()), ex);
            }
            catch (Exception ex2) {
                throw new IllegalArgumentException(String.format("Unexpected exception %s while attempting to construct a new instance of %s", ex2.getClass().getName(), loader.getName()), ex2);
            }
            final Pattern[] arr$;
            final Pattern[] patterns = arr$ = instance.getPluginFileFilters();
            for (final Pattern pattern : arr$) {
                this.fileAssociations.put(pattern, instance);
            }
            return;
        }
        throw new IllegalArgumentException(String.format("Class %s does not implement interface PluginLoader", loader.getName()));
    }
    
    public Plugin[] loadPlugins(final File directory) {
        final List<Plugin> result = new ArrayList<Plugin>();
        final File[] arr$;
        final File[] files = arr$ = directory.listFiles();
        for (final File file : arr$) {
            Plugin plugin = null;
            try {
                plugin = this.loadPlugin(file);
            }
            catch (InvalidPluginException ex) {
                this.server.getLogger().log(Level.SEVERE, "Could not load " + file.getPath() + " in " + directory.getPath() + ": " + ex.getMessage(), ex);
            }
            catch (InvalidDescriptionException ex2) {
                this.server.getLogger().log(Level.SEVERE, "Could not load " + file.getPath() + " in " + directory.getPath() + ": " + ex2.getMessage(), ex2);
            }
            if (plugin != null) {
                result.add(plugin);
            }
        }
        return result.toArray(new Plugin[result.size()]);
    }
    
    public Plugin loadPlugin(final File file) throws InvalidPluginException, InvalidDescriptionException {
        final Set<Pattern> filters = this.fileAssociations.keySet();
        Plugin result = null;
        for (final Pattern filter : filters) {
            final String name = file.getName();
            final Matcher match = filter.matcher(name);
            if (match.find()) {
                final PluginLoader loader = this.fileAssociations.get(filter);
                result = loader.loadPlugin(file);
            }
        }
        if (result != null) {
            this.plugins.add(result);
            this.lookupNames.put(result.getDescription().getName(), result);
        }
        return result;
    }
    
    public Plugin getPlugin(final String name) {
        return this.lookupNames.get(name);
    }
    
    public Plugin[] getPlugins() {
        return this.plugins.toArray(new Plugin[0]);
    }
    
    public boolean isPluginEnabled(final String name) {
        final Plugin plugin = this.getPlugin(name);
        return this.isPluginEnabled(plugin);
    }
    
    public boolean isPluginEnabled(final Plugin plugin) {
        return plugin != null && this.plugins.contains(plugin) && plugin.isEnabled();
    }
    
    public void enablePlugin(final Plugin plugin) {
        if (!plugin.isEnabled()) {
            plugin.getPluginLoader().enablePlugin(plugin);
        }
    }
    
    public void disablePlugins() {
        for (final Plugin plugin : this.getPlugins()) {
            this.disablePlugin(plugin);
        }
    }
    
    public void disablePlugin(final Plugin plugin) {
        if (plugin.isEnabled()) {
            plugin.getPluginLoader().disablePlugin(plugin);
            this.server.getScheduler().cancelTasks(plugin);
        }
    }
    
    public void clearPlugins() {
        synchronized (this) {
            this.disablePlugins();
            this.plugins.clear();
            this.lookupNames.clear();
            this.listeners.clear();
        }
    }
    
    public void callEvent(final Event event) {
        final SortedSet<RegisteredListener> eventListeners = this.listeners.get(event.getType());
        if (eventListeners != null) {
            for (final RegisteredListener registration : eventListeners) {
                try {
                    registration.callEvent(event);
                }
                catch (Throwable ex) {
                    this.server.getLogger().log(Level.SEVERE, "Could not pass event " + event.getType() + " to " + registration.getPlugin().getDescription().getName(), ex);
                }
            }
        }
    }
    
    public void registerEvent(final Event.Type type, final Listener listener, final Event.Priority priority, final Plugin plugin) {
        if (!plugin.isEnabled()) {
            this.server.getLogger().warning("Plugin '" + plugin.getDescription().getName() + "' (ver " + plugin.getDescription().getVersion() + ") is registering events before it is enabled. It may be misbehaving and the author needs to fix this.");
        }
        this.getEventListeners(type).add(new RegisteredListener(listener, priority, plugin, type));
    }
    
    public void registerEvent(final Event.Type type, final Listener listener, final EventExecutor executor, final Event.Priority priority, final Plugin plugin) {
        this.getEventListeners(type).add(new RegisteredListener(listener, executor, priority, plugin));
    }
    
    private SortedSet<RegisteredListener> getEventListeners(final Event.Type type) {
        SortedSet<RegisteredListener> eventListeners = this.listeners.get(type);
        if (eventListeners != null) {
            return eventListeners;
        }
        eventListeners = new TreeSet<RegisteredListener>(this.comparer);
        this.listeners.put(type, eventListeners);
        return eventListeners;
    }
}
