// 
// Decompiled by Procyon v0.5.36
// 

package org.yaml.snakeyaml.reader;

import java.nio.charset.Charset;
import java.io.IOException;
import org.yaml.snakeyaml.error.YAMLException;
import org.yaml.snakeyaml.scanner.Constant;
import org.yaml.snakeyaml.error.Mark;
import java.util.regex.Matcher;
import java.io.Reader;
import java.util.regex.Pattern;

public class StreamReader
{
    static final Pattern NON_PRINTABLE;
    private String name;
    private final Reader stream;
    private int pointer;
    private boolean eof;
    private final StringBuilder buffer;
    private int index;
    private int line;
    private int column;
    
    public StreamReader(final String stream) {
        this.pointer = 0;
        this.eof = true;
        this.index = 0;
        this.line = 0;
        this.column = 0;
        this.name = "<string>";
        this.buffer = new StringBuilder();
        this.checkPrintable(stream);
        this.buffer.append(stream);
        this.stream = null;
        this.eof = true;
    }
    
    public StreamReader(final Reader reader) {
        this.pointer = 0;
        this.eof = true;
        this.index = 0;
        this.line = 0;
        this.column = 0;
        this.name = "<reader>";
        this.buffer = new StringBuilder();
        this.stream = reader;
        this.eof = false;
    }
    
    void checkPrintable(final CharSequence data) {
        final Matcher em = StreamReader.NON_PRINTABLE.matcher(data);
        if (em.find()) {
            final int position = this.index + this.buffer.length() - this.pointer + em.start();
            throw new ReaderException(this.name, position, em.group().charAt(0), "special characters are not allowed");
        }
    }
    
    public Mark getMark() {
        return new Mark(this.name, this.index, this.line, this.column, this.buffer.toString(), this.pointer);
    }
    
    public void forward() {
        this.forward(1);
    }
    
    public void forward(final int length) {
        if (this.pointer + length + 1 >= this.buffer.length()) {
            this.update(length + 1);
        }
        char ch = '\0';
        for (int i = 0; i < length; ++i) {
            ch = this.buffer.charAt(this.pointer);
            ++this.pointer;
            ++this.index;
            if (Constant.LINEBR.has(ch) || (ch == '\r' && this.buffer.charAt(this.pointer) != '\n')) {
                ++this.line;
                this.column = 0;
            }
            else if (ch != '\ufeff') {
                ++this.column;
            }
        }
    }
    
    public char peek() {
        return this.peek(0);
    }
    
    public char peek(final int index) {
        if (this.pointer + index + 1 > this.buffer.length()) {
            this.update(index + 1);
        }
        return this.buffer.charAt(this.pointer + index);
    }
    
    public String prefix(final int length) {
        if (this.pointer + length >= this.buffer.length()) {
            this.update(length);
        }
        if (this.pointer + length > this.buffer.length()) {
            return this.buffer.substring(this.pointer, this.buffer.length());
        }
        return this.buffer.substring(this.pointer, this.pointer + length);
    }
    
    private void update(final int length) {
        this.buffer.delete(0, this.pointer);
        this.pointer = 0;
        while (this.buffer.length() < length) {
            String rawData = null;
            if (!this.eof) {
                final char[] data = new char[1024];
                int converted = -2;
                try {
                    converted = this.stream.read(data);
                }
                catch (IOException ioe) {
                    throw new YAMLException(ioe);
                }
                if (converted == -1) {
                    this.eof = true;
                }
                else {
                    rawData = new String(data, 0, converted);
                }
            }
            if (rawData != null) {
                this.checkPrintable(rawData);
                this.buffer.append(rawData);
            }
            if (this.eof) {
                this.buffer.append('\0');
                break;
            }
        }
    }
    
    public int getColumn() {
        return this.column;
    }
    
    public Charset getEncoding() {
        return Charset.forName(((UnicodeReader)this.stream).getEncoding());
    }
    
    public int getIndex() {
        return this.index;
    }
    
    public int getLine() {
        return this.line;
    }
    
    static {
        NON_PRINTABLE = Pattern.compile("[^\t\n\r -~\u0085 -\ud7ff\ue000-\ufffc]");
    }
}
